//************************************************************************
// T6963.C
// LowLevel Routinen fr LCD-Displays mit T6963
//************************************************************************

#include	"T6963.h"
#include <avr/pgmspace.h>
unsigned char LCD_STATUS;       	// Status-Byte

/**** LCD-Display-Routinen **********************************************/
void lcd_init(void)
        {
        unsigned int t;
		LCD_CTRL = 0xff;
		DDRC=0xff;
		PORTD=0x00;
		DDRD=0xFF;
		LCD_FS_H();
		LCD_RV_L();
        LCD_RST_L();          // Display Reset
                for (t=0; t<1000; t++)
                {}              // ca. 125s. Reset
        LCD_RST_H();          // Display Reset fertig
		
        lcd_send_data(T_BASE%256);        // Text Adresse
        lcd_send_data(T_BASE>>8);
        lcd_send_cmd(0x40);
        lcd_send_data(BYTES_PER_ROW%256);  // Zeilenlnge
        lcd_send_data(BYTES_PER_ROW>>8);
        lcd_send_cmd(0x41);
        lcd_send_data(G_BASE%256);        // Grafik Adresse
        lcd_send_data(G_BASE>>8);
        lcd_send_cmd(0x42);
        lcd_send_data(BYTES_PER_ROW%256);        // Zeichenlnge
        lcd_send_data(BYTES_PER_ROW>>8);
        lcd_send_cmd(0x43);
        lcd_send_data(0x02);	// Offset Pointer
        lcd_send_data(0x00);
        lcd_send_cmd(0x22);
        lcd_send_cmd(0x80);			// OR-Mode
        lcd_send_cmd(0x9C);			// Text an Grafik an
        lcd_send_data(0x00);        // Adress Pointer setzen
        lcd_send_data(0x00);
        lcd_send_cmd(0x24);
        lcd_clear_ram();        // zum Schlu noch das Ram des
        }                       // Display's lschen

void lcd_status(void)
        {
        DATA_DIR_IN();         
        LCD_CD_H();             // Commando High
        LCD_RD_L();             // Read Low
        LCD_CE_L();             // Chip Enable Low
        LCD_STATUS = LCD_READ;  // Status lesen
        LCD_CE_H();             // Chip Enable High
		LCD_RD_H();             // Read High
        LCD_STATUS &= (0x03);   // Bits ausmaskieren
        }

void lcd_status_am(void)
        {
        DATA_DIR_IN();         
        LCD_CD_H();             // Commando High
        LCD_RD_L();             // Read Low
        LCD_CE_L();             // Chip Enable Low
        LCD_STATUS = LCD_READ;  // Status lesen
        LCD_CE_H();             // Chip Enable High
		LCD_RD_H();             // Read High
        LCD_STATUS &= (0x08);   // Bits ausmaskieren
        }

void lcd_send_cmd(char cmd)         // Commando an Display senden        
        {
            do
                {
                  lcd_status();
                }
            while (LCD_STATUS != 0x03);
        DATA_DIR_OUT();             // Port 2 als Ausgang
        LCD_WRITE = cmd;
        LCD_CD_H();             // Commando High
        LCD_WR_L();             // Write Low
        LCD_CE_L();             // Chip Enable Low
        LCD_CE_H();             // Chip Enable High
        LCD_WR_H();             // Write High
        }

void lcd_send_data(char data)         // Daten an Display senden        
        {
            do
                {
                  lcd_status();
                }
            while (LCD_STATUS != 0x03);
        DATA_DIR_OUT();             // Port 2 als Ausgang
        LCD_WRITE = data;
        LCD_CD_L();             // Commando High
        LCD_WR_L();             // Write Low
        LCD_CE_L();             // Chip Enable Low
        LCD_CE_H();             // Chip Enable High
        LCD_WR_H();             // Write High
        }

void lcd_send_data_am(char data)         // Daten an Display senden
        {
            do
                {
                  lcd_status_am();
                }
            while (LCD_STATUS != 0x08);
        DATA_DIR_OUT();             // Port 2 als Ausgang
        LCD_WRITE = data;
        LCD_CD_L();             // Daten Low
        LCD_WR_L();             // Write Low
        LCD_CE_L();             // Chip Enable Low
        LCD_CE_H();             // Chip Enable High
        LCD_WR_H();             // Write High
        }

void lcd_printp(const char *progmem_s, char adress_h, char adress_l)
        {
        char c;
        lcd_send_data(adress_l);        
        lcd_send_data(adress_h);
        lcd_send_cmd(0x24);
        lcd_send_cmd(0xB0);
		while ( (c = pgm_read_byte(progmem_s++)))
			{
			lcd_send_str(c);
			}
        lcd_send_cmd(0xB2);
        }

		
void lcd_print(char *txt, char adress_h, char adress_l)
        {
        char c;
		lcd_send_data(adress_l);        
        lcd_send_data(adress_h);
        lcd_send_cmd(0x24);
        while((c = *txt)) //Solange Zeichen grer 0x00
			{
			txt++; //Nchstes Zeichen
			if (c == 0) break; //Raus wenn Zeichen gleich 0
			lcd_send_str(c); 
			lcd_send_cmd(0xc0);
			}
        }
		
void lcd_send_str(unsigned char lcd_dat)
        {
        lcd_dat -= 0x20;        // 20h subtrahieren da Display nich ASCII
        lcd_send_data_am(lcd_dat);     // Daten an Display senden
        }

void lcd_clear_ram(void)
        {
        unsigned int t;
        lcd_send_data(0x00);        // laut Datenblatt des Herstellers 8kb
        lcd_send_data(0x00);
        lcd_send_cmd(0x24);
        lcd_send_cmd(0xB0);
                for (t=0; t<8192; t++)
                {
                lcd_send_data_am(0x00);     // Daten an Display senden
                }
        lcd_send_cmd(0xB2);
        }
void lcd_printg(const char *progmem_s, char adress_h, char adress_l) 
	{ 
	unsigned int length =5120; //< dieses ist die Bildgre in Bytes 
	unsigned char c; 
	lcd_send_data(adress_l); 
	lcd_send_data(adress_h); 
	lcd_send_cmd(0x24); 
	lcd_send_cmd(0xB0); 

	while (length !=0)         //es wird ein Bild in der Gre 128*64 Pixel
        {                   //mit Font 6*8 angezeigt
		c = pgm_read_byte(progmem_s++);
		lcd_send_strg(c);
		length--;
		}
	lcd_send_cmd(0xB2); 
	} 

void lcd_send_strg(unsigned char lcd_dat) 
	{ 
	lcd_send_data(lcd_dat); // Daten an Display senden 
	} 
	
void lcd_setpixel(int x, int y)  
{
	int addr;       // memory address of byte containing pixel to write

  addr =  G_BASE + (y*BYTES_PER_ROW)  + (x/6);
  lcd_send_data(addr%256); 
  lcd_send_data(addr>>8); 
  lcd_send_cmd(0x24);  // set LCD addr. pointer
  lcd_send_cmd(0xf8 | (5-(x%6)) );  // set bit-within-byte command

} // end lcd_setpixel()
/*****************************************************************************
Name:	Int To Ascii Hex   
Parameters:	
		dest_string
			Pointer to a buffer will the string will reside
		min_digits
			Specifies the minimum number of characters the output string will
			have. Leading zeros will be written as '0' characters.
Returns:
		A pointer to the string's NULL character in the string that was just
		created.
Description: 
		This function is used to convert a passed unsigned int into a ASCII
		string represented in Hexidecimal format.
*****************************************************************************/
char * IntToAsciiHex(char * dest_string,int min_digits,unsigned int value)
{
	unsigned int i, total_digits = 0;
	char buff[4];
	
	for(i=0;i<4;i++)
	{
		buff[i] = (char)(value & 0x0F);
		value = value >> 4;
		if( buff[i] <= 9)
			buff[i] += '0';
		else
			buff[i] = (char)(buff[i] - 0xA + 'A');

		if(buff[i] != '0')
			total_digits = i+1;
	}

	if( total_digits < 	min_digits)
		total_digits = min_digits;

	i = total_digits;
	while(i)
	{
		*dest_string++ = buff[i-1];
		i--;
	}

	*dest_string = 0;

	return dest_string;
}
/*****************************************************************************
Name:	Int To Ascii Dec   
Parameters:	
		dest_string
			Pointer to a buffer will the string will reside
		min_digits
			Specifies the minimum number of characters the output string will
			have. Leading zeros will be written as '0' characters.
Returns:
		A pointer to the string's NULL character in the string that was just
		created.
Description: 
		This function is used to convert a passed unsigned int into a ASCII
		string represented in base 10 decimal format.
*****************************************************************************/
char * IntToAsciiDec(char * dest_string,int min_digits,unsigned int value)
{
	const unsigned long base10[] = {1,10,100,1000,10000,100000};

	unsigned int tmp;
	unsigned int i, total_digits = 0;
	char buff[5];
	
	for(i=0;i<5;i++)
	{
		tmp = (int)( value % base10[i+1] );
		value -= tmp;

		buff[i] = (char)( tmp / base10[i] );
		buff[i] += '0';
		if(buff[i] != '0')
			total_digits = i+1;
	}
	if( total_digits < 	min_digits)
		total_digits = min_digits;
	i = total_digits;
	while(i)
	{
		*dest_string++ = buff[i-1];
		i--;
	}
	*dest_string = 0;
	return dest_string;
}
